% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_generation.R
\name{data_generation}
\alias{data_generation}
\title{Data Generation Program}
\usage{
data_generation(random_seed, N, sigma, sigma_u)
}
\arguments{
\item{random_seed}{a single random seed for reproducibility. The datatype is \code{integer}.}

\item{N}{the total number of subjects to simulate. The datatype is \code{integer}.}

\item{sigma}{the global error standard deviation. The datatype is \code{double}.}

\item{sigma_u}{the standard deviation of the subject-level random effects. The datatype is \code{double}.}
}
\value{
\code{data_generation} returns a simulated \code{data.frame}.
}
\description{
Generates simulated data for evaluating the causal inference models.
}
\examples{
df1 <- data_generation(random_seed = 100,
                       N = 100,
                       sigma = 0.2,
                       sigma_u = 0.1)

print(head(df1))
}
